import datetime
import hashlib
import time
import uuid
from functools import wraps

from app.settings import SECRET_KEY, SIMPLE_JWT

cache_data = {}


def cache(lifetime=300):
    '''
    装饰器：系统缓存
    @Cache(lifetime=60)
    lifetime单位秒，缓存时间
    '''

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            key = get_md5(func.__module__ + func.__name__)
            if cache_data.get(key):
                delta = datetime.timedelta(seconds=cache_data[key]['lifetime'])
                now = datetime.datetime.now()
                if compare_datetime(cache_data[key]['createtime'] + delta, now):
                    return cache_data[key]['data']
            data = func(*args, **kwargs)
            cache_data[key] = {'lifetime': lifetime, 'createtime': datetime.datetime.now(), 'data': data}
            return data

        return wrapper

    return decorator


def get_md5(s):
    m = hashlib.md5()
    m.update(s.encode(encoding='UTF-8'))
    return m.hexdigest()


def get_uuid():
    return uuid.uuid4().hex


def get_timestamp():
    return int(time.time())


def compare_datetime(d1, d2):
    if d1 > d2:
        return True
    elif d1 == d2:
        return None
    else:
        return False


def get_auth(content='', secret=SECRET_KEY):
    timestamp = get_timestamp()
    skey = content + secret + str(timestamp)
    return hashlib.sha256(skey.encode()).hexdigest(), timestamp


def verify_auth(auth, timestamp, content='', secret=SECRET_KEY):
    delta = SIMPLE_JWT['ACCESS_TOKEN_LIFETIME']
    now = datetime.datetime.now()
    t = datetime.datetime.fromtimestamp(timestamp)
    if compare_datetime(now, t + delta):
        return None  # token过期
    skey = content + secret + str(timestamp)
    if auth == hashlib.sha256(skey.encode(encoding='UTF-8')).hexdigest():
        return True  # token验证通过
    else:
        return False  # token验证失败


def build_tree(data):
    node_dict = {item['id']: item for item in data}
    node_list = []
    for item in data:
        item.setdefault('children', [])
        parent_item = node_dict.get(item['parent'])
        if parent_item:
            children = parent_item.setdefault('children', [])
            children.append(item)
        else:
            node_list.append(item)
    return node_list
